const http = require('http');
const { Semantics, Semantic } = require('./src/semantics');
const { Generators, Generator } = require('./src/generators');
const { Config } = require('./src/config');
const fetch = require('node-fetch')

const process = (url, key, config) => {
  let data = Object.assign( { key, version: '3' }, config.config );
  const generators = new Generators(data.generators.map( (g) => new Generator(g[0], g[1]) ) )
  delete data['generators']
  const semantics = new Semantics(data.semantics.map( (g) => new Semantic(g[0], g[1]) ) )
  delete data['semantics']

  // timeout: 1000 * 60 * 3,
  return fetch(`${url}/process`, {
    method: "POST",
    body: JSON.stringify(data),
    timeout: 1000 * 60 * 3,
    headers: {
      mode: "no-cors",
      "Content-Type": "application/json"
    },
    }).then( result => result.json() )
      .then( json => {
      if (json.status != 200) {
        throw `Error ${json.status} - ${json.error}`;
        return;
      }
      if (json.results) {
        json.results = json.results.map( (result) => semantics.apply(config.config.objects, result) )
        json.generated = json.results.map( (result) => generators.apply(result) )
      }
      return json;
    });
}

module.exports = { process }
